@@ -1,7 +1,7 @@
 # Accelerator for pip, the Python package manager.
 #
 # Author: Peter Odding <peter.odding@paylogic.eu>
-# Last Change: July 21, 2013
+# Last Change: July 25, 2013
 # URL: https://github.com/paylogic/pip-accel
 #
 # TODO Permanently store logs in the pip-accel directory (think about log rotation).
@@ -21,9 +21,10 @@ taking a look at the following functions:
 """
 
 # Semi-standard module versioning.
-__version__ = '0.9.10'
+__version__ = '0.9.12'
 
 # Standard library modules.
+import functools
 import os
 import os.path
 import pkg_resources
@@ -44,10 +45,11 @@ from pip_accel.deps import sanity_check_dependencies
 from pip_accel.logger import logger
 
 # External dependencies.
+from pip import parseopts
 from pip.backwardcompat import string_types
-from pip.baseparser import create_main_parser
+from pip.cmdoptions import requirements as requirements_option
 from pip.commands.install import InstallCommand
-from pip.exceptions import DistributionNotFound, InstallationError
+from pip.exceptions import DistributionNotFound, InstallationError, PreviousBuildDirError
 from pip.log import logger as pip_logger
 from pip.status_codes import SUCCESS
 
@@ -123,15 +125,18 @@ def main():
         sys.exit(1)
     main_timer = Timer()
     initialize_directories()
+    build_directory = tempfile.mkdtemp()
     # Execute "pip install" in a loop in order to retry after intermittent
     # error responses from servers (which can happen quite frequently).
     try:
         for i in xrange(1, MAX_RETRIES):
             try:
-                requirements = unpack_source_dists(arguments)
+                unpack_partial = functools.partial(unpack_source_dists, arguments, build_directory)
+                requirements = previous_build_workaround(unpack_partial, build_directory)
             except DistributionNotFound:
                 logger.warn("We don't have all source distributions yet!")
-                download_source_dists(arguments)
+                download_partial = functools.partial(download_source_dists, arguments, build_directory)
+                previous_build_workaround(download_partial, build_directory)
             else:
                 if not requirements:
                     logger.info("No unsatisfied requirements found, probably there's nothing to do.")
@@ -146,6 +151,9 @@ def main():
         # Abort early when pip reports installation errors.
         logger.fatal("pip reported unrecoverable installation errors. Please fix and rerun!")
         sys.exit(1)
+    finally:
+        # Always cleanup temporary build directory.
+        shutil.rmtree(build_directory)
     # Abort when after N retries we still failed to download source distributions.
     logger.fatal("External command failed %i times, aborting!" % MAX_RETRIES)
     sys.exit(1)
@@ -167,7 +175,28 @@ def print_usage():
         at https://github.com/paylogic/pip-accel
     """).strip()
 
-def unpack_source_dists(arguments):
+def previous_build_workaround(partial_function, build_directory):
+    """
+    Since pip 1.4, pip refuses to touch existing build directories, even if pip
+    itself created those build directories! This function implements a brute
+    force workaround until I find a better way.
+
+    :param partial_function: The function to apply the workaround to.
+    :param build_directory: The pathname of the build directory.
+    :returns: The return value of the partial function.
+    """
+    try:
+        return partial_function()
+    except PreviousBuildDirError:
+        logger.warn("Caught a 'previous build directory' error, applying workaround ..")
+        # This sucks but I don't see any way around it. Shouldn't pip try to
+        # prevent this situation from happening? We always start from a clean
+        # build directory anyway :-s.
+        shutil.rmtree(build_directory)
+        os.makedirs(build_directory)
+        return partial_function()
+
+def unpack_source_dists(arguments, build_directory):
     """
     Check whether there are local source distributions available for all
     requirements, unpack the source distribution archives and find the names
@@ -191,12 +220,13 @@ def unpack_source_dists(arguments):
     logger.info("Unpacking local source distributions ..")
     # Execute pip to unpack the source distributions.
     requirement_set = run_pip(arguments + ['--no-install'],
-                              use_remote_index=False)
+                              use_remote_index=False,
+                              build_directory=build_directory)
     logger.info("Unpacked local source distributions in %s.", unpack_timer)
     requirements = []
     for install_requirement in sorted_requirements(requirement_set):
         if install_requirement.satisfied_by:
-          logger.info("Requirement already satisfied: %s.", install_requirement)
+            logger.info("Requirement already satisfied: %s.", install_requirement)
         else:
             req = ensure_parsed_requirement(install_requirement)
             requirements.append((req.project_name,
@@ -234,7 +264,7 @@ def ensure_parsed_requirement(install_requirement):
         req = pkg_resources.Requirement.parse(req)
     return req
 
-def download_source_dists(arguments):
+def download_source_dists(arguments, build_directory):
     """
     Download missing source distributions.
 
@@ -244,8 +274,11 @@ def download_source_dists(arguments):
     logger.info("Downloading source distributions ..")
     # Execute pip to download missing source distributions.
     try:
-        run_pip(arguments + ['--no-install'], use_remote_index=True)
+        run_pip(arguments + ['--no-install'], use_remote_index=True, build_directory=build_directory)
         logger.info("Finished downloading source distributions in %s.", download_timer)
+    except PreviousBuildDirError:
+        # Shouldn't be swallowed.
+        raise
     except Exception, e:
         logger.warn("pip raised an exception while downloading source distributions: %s.", e)
 
@@ -508,7 +541,7 @@ def fix_hashbang(python, contents):
         logger.debug("Warning: Failed to match hashbang: %r.", hashbang)
     return contents
 
-def run_pip(arguments, use_remote_index):
+def run_pip(arguments, use_remote_index, build_directory=None):
     """
     Execute a modified ``pip install`` command. This function assumes that the
     arguments concern a ``pip install`` command (:py:func:`main()` makes sure
@@ -528,6 +561,8 @@ def run_pip(arguments, use_remote_index):
             command_line += ['pip'] + arguments[:i+1] + [
                     '--download-cache=%s' % download_cache,
                     '--find-links=file://%s' % source_index]
+            if build_directory:
+                command_line += ['--build-directory=%s' % build_directory]
             if not use_remote_index:
                 command_line += ['--no-index']
             command_line += arguments[i+1:]
@@ -535,10 +570,11 @@ def run_pip(arguments, use_remote_index):
     else:
         command_line = ['pip'] + arguments
     logger.info("Executing command: %s", ' '.join(command_line))
-    parser = create_main_parser()
+    # XXX Nasty hack required for pip 1.4 compatibility (workaround for global state).
+    requirements_option.default = []
+    cmd_name, options, args, parser = parseopts(command_line[1:])
     pip = CustomInstallCommand(parser)
-    initial_options, args = parser.parse_args(command_line[1:])
-    exit_status = pip.main(command_line[2:], initial_options)
+    exit_status = pip.main(args[1:], options)
     # Make sure the output of pip and pip-accel are not intermingled.
     sys.stdout.flush()
     update_source_dists_index()
