@@ -1,7 +1,7 @@
 # Logging for the pip accelerator.
 #
 # Author: Peter Odding <peter.odding@paylogic.eu>
-# Last Change: June 15, 2013
+# Last Change: July 21, 2013
 # URL: https://github.com/paylogic/pip-accel
 
 """
@@ -13,14 +13,14 @@ import logging
 import os
 import sys
 
-# External dependencies.
-from coloredlogs import ColoredStreamHandler
+# External dependency.
+import coloredlogs
+coloredlogs.install()
 
 # Initialize the logging subsystem.
 logger = logging.getLogger('pip-accel')
-logger.setLevel(logging.INFO)
-logger.addHandler(ColoredStreamHandler())
+logger.setLevel(logging.DEBUG)
 
 # Check if the operator requested verbose output.
 if '-v' in sys.argv or 'PIP_ACCEL_VERBOSE' in os.environ:
-    logger.setLevel(logging.DEBUG)
+    logging.getLogger().setLevel(logging.DEBUG)
